/*
 * devices.c - dump a list of reserved DOS devices
 * Copyright (c) 2005-2021, D.C. van Moolenbroek, XISE <dc@xise.nl>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 *  * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

void printf(TCHAR *fmt, ...) {
  TCHAR buf[1024];
  va_list va;
  DWORD written;

  va_start(va, fmt);
  wvsprintf(buf, fmt, va);
  buf[sizeof(buf)/sizeof(buf[0])-1] = 0;
  va_end(va);

  // FIXME: with UNICODE this needs a better solution, because apparently
  // WriteConsole() does not work with output redirection to a file.
  // See also https://docs.microsoft.com/en-us/windows/console/writeconsole .
  WriteFile(GetStdHandle(STD_OUTPUT_HANDLE), buf, lstrlen(buf), &written,
   NULL);
}

int WINAPI WinMain(HINSTANCE hinst, HINSTANCE hprev, LPSTR cmdline, int show) {
  TCHAR *buf;
  DWORD size, used, pos;
  int i = 0;

  size = 16384;
  buf = LocalAlloc(LMEM_FIXED, size * sizeof(TCHAR));

  for (;;) {
    if (buf == NULL) {
      printf(TEXT("Error allocating memory\n"));
      return 1;
    }

    used = QueryDosDevice(NULL, buf, size);
    if (used > 0)
      break;

    if (GetLastError() != ERROR_INSUFFICIENT_BUFFER) {
      printf(TEXT("Error obtaining list\n"));
      return 1;
    }

    size *= 2;
    buf = LocalReAlloc(buf, size * sizeof(TCHAR), LMEM_MOVEABLE);
  }

  printf(TEXT("List of DOS devices:\n"));

  for (pos = 0; pos < used; pos += lstrlen(buf + pos) + 1)
   if (buf[pos])
    printf(TEXT("#%d: %s\n"), i++, buf + pos);

  return 0;
}
